# uwsgi --queue 10 --queue-store test.queue --master --module tests.queue --socket :3031

import uwsgi
import os
from flask import Flask, render_template, request, redirect, flash

app = Flask(__name__)
app.debug = True
app.secret_key = os.urandom(24)


@app.route('/')
def index():
    return render_template('queue.html', uwsgi=uwsgi)


@app.route('/push', methods=['POST'])
def push_item():
    if uwsgi.queue_push(request.form['body']):
        flash('item enqueued')
        return redirect('/')
    else:
        flash('unable to enqueue item')
        return render_template('queue.html', uwsgi=uwsgi)


@app.route('/get', methods=['POST'])
def get_item():
    flash("slot %s value = %s" % (request.form['slot'], uwsgi.queue_get(int(request.form['slot']))))
    return redirect('/')


@app.route('/pop', methods=['POST'])
def pop_item():
    flash("popped value = %s" % uwsgi.queue_pop())
    return redirect('/')


@app.route('/pull', methods=['POST'])
def pull_item():
    flash("pulled value = %s" % uwsgi.queue_pull())
    return redirect('/')


@app.route('/set', methods=['POST'])
def set_item():
    if uwsgi.queue_set(int(request.form['pos']), request.form['body']):
        flash('item set')
        return redirect('/')
    else:
        flash('unable to set item')
        return render_template('queue.html', uwsgi=uwsgi)



"""
Greenlets support

simply --pypy-exec this file to enable PyPy's greenlet mode

Example:

uwsgi --http-socket :9090 --pypy-home /opt/pypy --pypy-wsgi werkzeug.testapp:test_app --pypy-exec contrib/pypy/uwsgi_pypy_greenlets.py --async 8
"""

import greenlet

uwsgi_pypy_greenlets = {}


def uwsgi_pypy_greenlet_wrapper():
    lib.async_schedule_to_req_green()


@ffi.callback("void()")
def uwsgi_pypy_greenlet_schedule():
    id = lib.uwsgi.wsgi_req.async_id
    modifier1 = lib.uwsgi.wsgi_req.uh.modifier1

    # generate a new greenlet
    if not lib.uwsgi.wsgi_req.suspended:
        uwsgi_pypy_greenlets[id] = greenlet.greenlet(uwsgi_pypy_greenlet_wrapper)
        lib.uwsgi.wsgi_req.suspended = 1

    # this is called in the main stack
    if lib.uwsgi.p[modifier1].suspend:
        lib.uwsgi.p[modifier1].suspend(ffi.NULL)

    # let's switch
    uwsgi_pypy_greenlets[id].switch()

    # back to the main stack
    if lib.uwsgi.p[modifier1].resume:
        lib.uwsgi.p[modifier1].resume(ffi.NULL)


@ffi.callback("void(struct wsgi_request *)")
def uwsgi_pypy_greenlet_switch(wsgi_req):
    wsgi_req.async_id
    modifier1 = wsgi_req.uh.modifier1

    # this is called in the current greenlet
    if lib.uwsgi.p[modifier1].suspend:
        lib.uwsgi.p[modifier1].suspend(wsgi_req)

    uwsgi_pypy_main_greenlet.switch()

    # back to the greenlet
    if lib.uwsgi.p[modifier1].resume:
        lib.uwsgi.p[modifier1].resume(wsgi_req)

    # update current running greenlet
    lib.uwsgi.wsgi_req = wsgi_req

if lib.uwsgi.async < 1:
    raise Exception("pypy greenlets require async mode !!!")
lib.uwsgi.schedule_to_main = uwsgi_pypy_greenlet_switch
lib.uwsgi.schedule_to_req = uwsgi_pypy_greenlet_schedule
uwsgi_pypy_main_greenlet = greenlet.getcurrent()



import time


def application(e, sr):
    time.sleep(3)
    print("3 seconds elapsed")
    sr('200 Ok', [('Content-Type', 'text/html')])

    time.sleep(2)
    print("2 seconds elapsed")

    yield "part1"

    try:
        time.sleep(2)
    except Exception:
        print("CLIENT DISCONNECTED !!!")
    print("2 seconds elapsed")

    yield "part2"

    try:
        time.sleep(2)
    except Exception:
        print("CLIENT DISCONNECTED !!!")
    print("2 seconds elapsed")

    yield "part3"

    time.sleep(2)
    print("2 seconds elapsed")

    yield "part4"

    print("end of request")



from threading import Thread
import uwsgi

from six.moves import queue


CONSUMERS = 4


def consumer(q):
    while True:
        item = q.get()
        print(item)
        # ... DO A HEAVY TASK HERE ...
        q.task_done()


def spawn_consumers():
    global q
    q = queue.Queue()
    for i in range(CONSUMERS):
        t = Thread(target=consumer, args=(q,))
        t.daemon = True
        t.start()
        print("consumer %d on worker %d started" % (i, uwsgi.worker_id()))


uwsgi.post_fork_hook = spawn_consumers


def application(env, start_response):
    global q

    # we pass a copy of the env dictionary as it gets cleared after yield/return
    q.put(env.copy())

    start_response('200 OK', [('Content-Type', 'text/html')])

    yield "Task enqueued"



import time
import uwsgi


def application(e, s):
    print("locking")
    uwsgi.lock()
    print("locked")
    time.sleep(3)
    uwsgi.unlock()
    print("UN-locked")
    s('200 OK', [('Content-Type', 'text/html')])
    return "slow"



#!./uwsgi --https :8443,foobar.crt,foobar.key --http-raw-body --gevent 100 --module tests.websockets_echo
import uwsgi
import time


def application(env, sr):

    ws_scheme = 'ws'
    if 'HTTPS' in env or env['wsgi.url_scheme'] == 'https':
        ws_scheme = 'wss'

    if env['PATH_INFO'] == '/':
        sr('200 OK', [('Content-Type', 'text/html')])
        return """
    <html>
      <head>
          <script language="Javascript">
            var s = new WebSocket("%s://%s/foobar/");
            s.onopen = function() {
              alert("connected !!!");
              s.send("ciao");
            };
            s.onmessage = function(e) {
        var bb = document.getElementById('blackboard')
        var html = bb.innerHTML;
        bb.innerHTML = html + '<br/>' + e.data;
            };

        s.onerror = function(e) {
            alert(e);
        }

    s.onclose = function(e) {
        alert("connection closed");
    }

            function invia() {
              var value = document.getElementById('testo').value;
              s.send(value);
            }
          </script>
     </head>
    <body>
        <h1>WebSocket</h1>
        <input type="text" id="testo"/>
        <input type="button" value="invia" onClick="invia();"/>
    <div id="blackboard" style="width:640px;height:480px;background-color:black;color:white;border: solid 2px red;overflow:auto">
    </div>
    </body>
    </html>
        """ % (ws_scheme, env['HTTP_HOST'])
    elif env['PATH_INFO'] == '/foobar/':
        uwsgi.websocket_handshake(env['HTTP_SEC_WEBSOCKET_KEY'], env.get('HTTP_ORIGIN', ''))
        print("websockets...")
        while True:
            msg = uwsgi.websocket_recv()
            uwsgi.websocket_send("[%s] %s" % (time.time(), msg))



import os
import subprocess


def application(env, start_response):

    boundary = 'uwsgi_mjpeg_frame'

    start_response('200 Ok', [
        ('Cache-Control', 'no-cache'),
        ('Cache-Control', 'private'),
        ('Pragma', 'no-cache'),
        ('Content-Type', 'multipart/x-mixed-replace; boundary=' + boundary),
    ])

    yield "--%s\r\n" % boundary

    while 1:
        yield "Content-Type: image/jpeg\r\n\r\n"
        print(subprocess.call('screencapture -t jpg -m -T 1 screenshot.jpg', shell=True))
        f = open('screenshot.jpg')
        yield env['wsgi.file_wrapper'](f)
        yield "\r\n--%s\r\n" % boundary
        # subprocess.call('./isightcapture -w 640 -h 480 screenshot.jpg', shell=True)



from flask import Flask, request
app = Flask(__name__)
app.debug = True


@app.route('/')
def hello_world():
    return "<form method='POST' enctype='multipart/form-data' action='/upload'><input type='text' name='pippo'/><input type='file' name='ufile'/><input type='submit' value='upload'/></form>"


@app.route('/upload',  methods=['GET', 'POST'])
def upload_file():
    return str(len(request.form.get('pippo', 'boh'))) + request.form.get('pippo', 'boh')
    return request.files['ufile'].read()



import os
import sys
try:
    from distutils import sysconfig
    paths = [
        sysconfig.get_python_inc(),
        sysconfig.get_python_inc(plat_specific=True),
    ]
except ImportError:
    import sysconfig
    paths = [
        sysconfig.get_path('include'),
        sysconfig.get_path('platinclude'),
    ]

os.environ['UWSGI_PYTHON_NOLIB'] = '1'

NAME = 'pyuwsgi'

CFLAGS = ['-I' + path for path in paths]
LDFLAGS = []
LIBS = []

GCC_LIST = ['pyuwsgi']



# uwsgi --spooler spool1 --spooler spool2 --spooler-cheap --spooler-frequency 5 --spooler-processes 4 --mule --shared-py-import=t/spooler/cheap.py --stats :5000
from uwsgidecorators import *
import time
import random
import os

def fake(args):
    time.sleep(6)
    return uwsgi.SPOOL_OK

uwsgi.spooler = fake

base = os.getcwd()
spoolers = [base + '/spool1', base + '/spool2']

@mule(1)
def spooler_enqueuer():
    while True:
        print("enqueuing task...")
        uwsgi.spool({'one':'two', 'spooler': random.choice(spoolers)})
        time.sleep(random.randrange(1, 15))



import uwsgi


def send_request(env, client):

    uwsgi.send(client, b"GET /intl/it_it/images/logo.gif HTTP/1.0\r\n")

    # test for suspend/resume
    uwsgi.suspend()

    uwsgi.send(client, b"Host: www.google.it\r\n\r\n")

    while 1:
        yield uwsgi.wait_fd_read(client, 2)
        if env['x-wsgiorg.fdevent.timeout']:
            return

        buf = uwsgi.recv(client, 4096)
        if buf:
            yield buf
        else:
            break


def application(env, start_response):

    c = uwsgi.async_connect('74.125.232.115:80')

    # wait for connection
    yield uwsgi.wait_fd_write(c, 2)

    if env['x-wsgiorg.fdevent.timeout']:
        uwsgi.close(c)
        raise StopIteration

    if uwsgi.is_connected(c):
        for r in send_request(env, c):
            yield r
    else:
        start_response('500 Internal Server Error', [('Content-Type', 'text/html')])
        yield "Internal Server Error"

    uwsgi.close(c)



#!/usr/bin/python3
# uwsgi --plugin python,http --http 0.0.0.0:8000 -w app

from werkzeug.wrappers import Request, Response


def application(env, start_response):
    request = Request(env)
    lines = b""
    while True:
        line = request.stream.readline()
        if not line:
            break;
        lines += line
    response = Response(lines)
    return response(env, start_response)



#! /usr/bin/env python
# coding = utf-8

from __future__ import print_function
from spooler_priority_constants import tasks, LOGFILE
from os import remove
import uwsgi

counter = 0


def spoolerHandler(env):
    global counter
    # Spooler is handling a task
    with open(LOGFILE, "a") as log:
        print("%s" % (env['name']), file=log)

    counter += 1

    if counter == len(tasks):
        # Each task has been processed.
        uwsgi.signal(17)

    # Spooler has done handling the task
    return uwsgi.SPOOL_OK

uwsgi.spooler = spoolerHandler

# Clear the logfile
try:
    remove(LOGFILE)
except OSError, e:  # log does not exist
    print(e)



# uwsgi --master --plugins=python27 --spooler=/var/spool/uwsgi/ --spooler-import spooler_dir.py
import uwsgi


def spooler_func(env):
    print(uwsgi.spooler_dir())
    return uwsgi.SPOOL_RETRY

uwsgi.spooler = spooler_func
uwsgi.spool({"foo": "bar"})



import uwsgi
import os
import gc
import sys
from uwsgidecorators import rpc, signal, postfork

from os.path import abspath, dirname, join

logo_png = abspath(join(dirname(__file__), "../logo_uWSGI.png"))

print(sys.version)
print(sys.version_info)
if "set_debug" in gc.__dict__:
    gc.set_debug(gc.DEBUG_SAVEALL)

print(os.environ)
print(sys.modules)
print(sys.argv)

try:
    DEBUG = sys.argv[1] == "debug"
except IndexError:
    DEBUG = False


def after_request_hook():
    print("request finished")


uwsgi.after_req_hook = after_request_hook


@rpc(b"hello")
def hello_rpc(one, two, three):
    arg0 = one[::-1]
    arg1 = two[::-1]
    arg2 = three[::-1]
    return "!%s-%s-%s!" % (arg1, arg2, arg0)


@signal(17)
def ciao_mondo(signum):
    print("Hello World")


def xsendfile(e, sr):
    sr(
        "200 OK", [("Content-Type", "image/png"), ("X-Sendfile", logo_png),],
    )
    return []


def serve_logo(e, sr):
    # use raw facilities (status will not be set...)
    uwsgi.send(
        b"%s 200 OK\r\nContent-Type: image/png\r\n\r\n"
        % e["SERVER_PROTOCOL"].encode("latin1")
    )
    uwsgi.sendfile(logo_png)
    return []


def serve_config(e, sr):
    sr("200 OK", [("Content-Type", "text/html")])
    for key in uwsgi.opt.keys():
        opt = uwsgi.opt[key]
        if not isinstance(opt, bytes):
            opt = str(opt).encode("utf-8")
        yield b"<b>%s</b> = %s<br/>" % (key, opt)


routes = {}
routes["/xsendfile"] = xsendfile
routes["/logo"] = serve_logo
routes["/config"] = serve_config


@postfork
def setprocname():
    if uwsgi.worker_id() > 0:
        uwsgi.setprocname(b"i am the worker %d" % uwsgi.worker_id())


def application(env, start_response):
    try:
        uwsgi.mule_msg(env["REQUEST_URI"], 1)
    except Exception:
        pass

    req = uwsgi.workers()[uwsgi.worker_id() - 1]["requests"]

    uwsgi.setprocname(b"worker %d managed %d requests" % (uwsgi.worker_id(), req))

    try:
        gc.collect(2)
    except Exception:
        pass
    if DEBUG:
        print(env["wsgi.input"].fileno())

    if env["PATH_INFO"] in routes:
        return routes[env["PATH_INFO"]](env, start_response)

    if DEBUG:
        print(env["wsgi.input"].fileno())

    try:
        gc.collect(2)
    except Exception:
        pass

    if DEBUG:
        print(len(gc.get_objects()))

    workers = ""
    for w in uwsgi.workers():
        apps = '<table border="1"><tr><th>id</th><th>mountpoint</th><th>startup time</th><th>requests</th></tr>'
        for app in w["apps"]:
            apps += "<tr><td>%d</td><td>%s</td><td>%d</td><td>%d</td></tr>" % (
                app["id"],
                app["mountpoint"],
                app["startup_time"],
                app["requests"],
            )
        apps += "</table>"
        workers += """
<tr>
<td>%d</td><td>%d</td><td>%s</td><td>%d</td><td>%d</td><td>%d</td><td>%s</td>
</tr>
        """ % (
            w["id"],
            w["pid"],
            w["status"],
            w["running_time"] / 1000,
            w["avg_rt"] / 1000,
            w["tx"],
            apps,
        )

    output = """
<img src="{script_name}/logo"/> version %s running on %s (remote user: %s)<br/>
<hr size="1"/>

Configuration<br/>
<iframe src="{script_name}/config"></iframe><br/>

<br/>
Workers and applications<br/>
<table border="1">
<tr>
<th>wid</th><th>pid</th><th>status</th><th>running time</th><th>average</th><th>tx</th><th>apps</th>
</tr>
%s
</table>

    """ % (
        uwsgi.version,
        uwsgi.hostname,
        env.get("REMOTE_USER", "None"),
        workers,
    )

    start_response(
        "200 OK", [("Content-Type", "text/html"), ("Content-Length", str(len(output)))]
    )

    return [output.format(script_name=env["SCRIPT_NAME"]).encode("utf-8")]



"""
Regression test for #2056 - uwsgi.workers() leaking objects.
"""
import uwsgi
import gc


def application(env, start_response):
    gc.collect()
    start_objs = len(gc.get_objects())

    for i in range(200):
        workers = uwsgi.workers()
        assert workers, "none/empty uwsgi.workers() - " + repr(workers)
        for w in workers:
            assert w["apps"], "none/empty apps in worker dict: " + repr(w)

    gc.collect()
    end_objs = len(gc.get_objects())
    diff_objs = end_objs - start_objs

    # Sometimes there is a spurious diff of 4 objects or so.
    if diff_objs > 10:
        start_response('500 Leaking', [('Content-Type', 'text/plain')])
        yield "Leaking objects...\n".encode("utf-8")
    else:
        start_response('200 OK', [('Content-Type', 'text/plain')])

    yield "{} {} {}\n".format(start_objs, end_objs, diff_objs).encode("utf-8")



# uwsgi --route-run remheader:Content-Length --route-run forcecl: --wsgi-file t/python/forcecl.py --http-socket :9090
def application(e, sr):
    sr('200 OK', [('Content-Length', '1'), ('Content-Length', '2')])
    return ['xxx']



from __future__ import print_function

from tasksconsumer import queueconsumer


@queueconsumer('fast', 4)
def fast_queue(arguments):
    print("fast", arguments)


@queueconsumer('slow')
def slow_queue(arguments):
    print("foobar", arguments)




import uwsgi

current_node = 0


def application(e, s):

    global current_node

    nodes = uwsgi.cluster_nodes()
    print(nodes)

    if len(nodes) == 0:
        print("no cluster node available")
        raise StopIteration

    if current_node >= len(nodes):
        current_node = 0

    node = nodes[current_node]

    for part in uwsgi.send_message(node, 0, 0, e, 0, e['wsgi.input'].fileno(), uwsgi.cl()):
        yield part

    current_node += 1



import uwsgi
import psycopg2


def ugreen_wait_callback(conn, timeout=-1):
    """A wait callback useful to allow uWSGI/uGreen to work with Psycopg."""
    while True:
        state = conn.poll()
        if state == psycopg2.extensions.POLL_OK:
            break
        elif state == psycopg2.extensions.POLL_READ:
            uwsgi.green_wait_fdread(conn.fileno())
        elif state == psycopg2.extensions.POLL_WRITE:
            uwsgi.green_wait_fdwrite(conn.fileno())
        else:
            raise Exception("Unexpected result from poll: %r", state)


# set the wait callback
psycopg2.extensions.set_wait_callback(ugreen_wait_callback)


def application(env, start_response):

    start_response('200 Ok', [('Content-type', 'text/html')])

    # connect
    conn = psycopg2.connect("dbname=prova user=postgres")
    # get cursor
    curs = conn.cursor()

    yield "<table>"

    # run query
    curs.execute("SELECT * FROM tests")

    while True:
        row = curs.fetchone()
        if not row:
            break
        yield "<tr><td>%s</td></tr>" % str(row)

    yield "</table>"

    conn.close()



from envoxy import View, Response, on, zmqc, auth_required

@on(endpoint='/v3/cards', protocols=['http'])
class CardsCollection(View):


    @auth_required(roles=['container', 'manager'])
    def get(self, request, **kwargs):
        return Response(
            zmqc.get(
                'muzzley-platform', 
                '/v3/data-layer/cards', 
                params=request.args,
                headers=request.headers.items()
            )
        )

    def post(self, request, **kwargs):
        return Response(
            zmqc.post(
                'muzzley-platform', 
                '/v3/data-layer/cards', 
                payload=request.get_json(),
                headers=request.headers.items()
            )
        )

@on(endpoint='/v3/cards/{uuid:str}', protocols=['http'])
class CardsDocument(View):

    def get(self, request, uuid, **kwargs):
        return Response(
            zmqc.get(
                'muzzley-platform', 
                '/v3/data-layer/cards/{}'.format(uuid),
                params=request.args,
                headers=request.headers.items()
            )
        )

    def put(self, request, uuid, **kwargs):
        return Response(
            zmqc.post(
                'muzzley-platform', 
                '/v3/data-layer/cards/{}'.format(uuid), 
                params=request.args,
                payload=request.get_json(),
                headers=request.headers.items()
            )
        )

    def patch(self, request, uuid, **kwargs):
        return Response(
            zmqc.patch(
                'muzzley-platform', 
                '/v3/data-layer/cards/{}'.format(uuid), 
                params=request.args,
                payload=request.get_json(),
                headers=request.headers.items()
            )
        )

    def delete(self, request, uuid, **kwargs):
        return Response(
            zmqc.delete(
                'muzzley-platform', 
                '/v3/data-layer/cards/{}'.format(uuid),
                headers=request.headers.items()
            )
        )





import uwsgi


def application(env, start_response):

    # open the socket
    fd = uwsgi.async_connect("192.168.173.100:3032")

    # wait for connection ready (3s timeout)
    yield uwsgi.wait_fd_write(fd, 3)

    # has timed out ?
    if env['x-wsgiorg.fdevent.timeout']:
        print("connection timed out !!!")
        uwsgi.close(fd)
        raise StopIteration

    # connection refused ?
    if not uwsgi.is_connected(fd):
        print("unable to connect")
        uwsgi.close(fd)
        raise StopIteration

    # send request
    # env can contains python objects, but send_message will discard them.
    # In this way we will automagically have a congruent and valid uwsgi packet
    uwsgi.async_send_message(fd, 0, 0, env)

    # send the http body
    # ready body in async mode and resend to fd
    # uwsgi.recv will use always an internal buffer of 4096, but can be limited in the number of bytes to read

    # does this request has a body ?
    cl = uwsgi.cl()

    if cl > 0:
        # get the input fd
        input = env['wsgi.input'].fileno()

        # read (in async mode) up to 'cl' data and send to uwsgi peer
        while cl > 0:
            bufsize = min(cl, 4096)
            yield uwsgi.wait_fd_read(input, 30)
            if env['x-wsgiorg.fdevent.timeout']:
                print("connection timed out !!!")
                uwsgi.close(fd)
                raise StopIteration
            body = uwsgi.recv(input, bufsize)
            if body:
                uwsgi.send(fd, body)
                cl = cl - len(body)
            else:
                break

    # wait for response (30s timeout)
    yield uwsgi.wait_fd_read(fd, 30)

    # has timed out ?
    if env['x-wsgiorg.fdevent.timeout']:
        print("connection timed out !!!")
        uwsgi.close(fd)
        raise StopIteration

    data = uwsgi.recv(fd)
    # recv the data, if it returns None the callable will end
    while data:
        yield data
        # wait for response
        yield uwsgi.wait_fd_read(fd, 30)
        if env['x-wsgiorg.fdevent.timeout']:
            print("connection timed out !!!")
            uwsgi.close(fd)
            raise StopIteration
        data = uwsgi.recv(fd)

    uwsgi.close(fd)



import os
NAME='router_basicauth'

CFLAGS = []
LDFLAGS = []
LIBS = []

# osx and openbsd do not need libcrypt
if os.uname()[0] not in ('Darwin', 'OpenBSD'):
    LIBS.append('-lcrypt')

GCC_LIST = ['router_basicauth']



import uwsgi
import time


def slow_task(args):
    time.sleep(10)
    return uwsgi.SPOOL_OK

uwsgi.spooler = slow_task


def application(env, start_response):

    name = uwsgi.spool({
        'Hello': 'World',
        'I am a': 'long running task'
    })
    print("spooled as %s" % name)

    start_response('200 Ok', [
        ('Content-Type', 'text/plain'),
        ('uWSGI-Status', 'spooled'),
    ])

    return "task spooled"



import uwsgi
import threading
import time
import sys


from six.moves import reload_module


def monitor1():
    while 1:
        time.sleep(1)
        print("i am the monitor 1")


def monitor2():
    while 1:
        time.sleep(2)
        print("i am the monitor 2")
        print(sys.modules)


def monitor3():
    while 1:
        time.sleep(5)
        print("5 seconds elapsed")
        # reload_module(fake)


def spawn_my_magic_threads():
    print("^^^ spawning magic threads ^^^")
    threading.Thread(target=monitor1).start()
    threading.Thread(target=monitor2).start()
    threading.Thread(target=monitor3).start()

uwsgi.post_fork_hook = spawn_my_magic_threads


def application(e, s):
    s('200 Ok', [('Content-Type', 'text/html')])
    return "Hello Threaded World !!!"



# uwsgi --async 1000 --ugreen
import uwsgi
import psycopg2


def async_wait(conn):
    # conn can be a connection or a cursor
    if not hasattr(conn, 'poll'):
        conn = conn.connection

    # interesting part: suspend until ready
    while True:
        state = conn.poll()
        if state == psycopg2.extensions.POLL_OK:
            break
        elif state == psycopg2.extensions.POLL_READ:
            uwsgi.wait_fd_read(conn.fileno())
            uwsgi.suspend()
        elif state == psycopg2.extensions.POLL_WRITE:
            uwsgi.wait_fd_write(conn.fileno())
            uwsgi.suspend()
        else:
            raise Exception("Unexpected result from poll: %r", state)


def application(env, start_response):

    start_response('200 Ok', [('Content-type', 'text/html')])

    conn = psycopg2.connect("dbname=template1 user=postgres", async=True)

    # suspend until connection
    async_wait(conn)

    curs = conn.cursor()

    yield "<table>"

    curs.execute("SELECT * FROM tests")

    # suspend until result
    async_wait(curs)

    while True:
        row = curs.fetchone()
        if not row:
            break
        yield "<tr><td>%s</td></tr>" % str(row)

    yield "</table>"

    conn.close()



import uwsgi
import socket
import errno


def send_request(env, client):

    client.setblocking(1)

    client.send(b"GET /intl/it_it/images/logo.gif HTTP/1.0\r\n")
    client.send(b"Host: www.google.it\r\n\r\n")

    uwsgi.green_schedule()

    while 1:
        uwsgi.green_wait_fdread(client.fileno(), 10)
        buf = client.recv(4096)
        if len(buf) == 0:
            break
        else:
            yield buf


def application(env, start_response):

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.setblocking(0)

    c = s.connect_ex(('74.125.77.104', 80))
    if c == errno.EINPROGRESS:
        uwsgi.green_wait_fdwrite(s.fileno(), 10)
        for r in send_request(env, s):
            yield r
    elif c == errno.EISCONN:
        for r in send_request(env, s):
            yield r
    else:
        uwsgi.green_schedule()
        start_response('500 Internal Server Error', [('Content-Type', 'text/html')])
        yield "Internal Server Error"

    s.close()



import psycopg2


def pg_wait(conn, env, timeout=0):
    while 1:
        state = conn.poll()
        if state == psycopg2.extensions.POLL_OK:
            raise StopIteration
        elif state == psycopg2.extensions.POLL_WRITE:
            yield env['x-wsgiorg.fdevent.writable'](conn.fileno(), timeout)
        elif state == psycopg2.extensions.POLL_READ:
            yield env['x-wsgiorg.fdevent.readable'](conn.fileno(), timeout)
        else:
            raise psycopg2.OperationalError("poll() returned %s" % state)


def application(env, start_response):

    start_response('200 OK', [('Content-Type', 'text/html')])

    connection = psycopg2.connect(database='uwsgi', user='uwsgi', password='uwsgi', host='192.168.173.100', async=1)

    for i in pg_wait(connection, env, 3):
        yield i

    print("connected")
    cursor = connection.cursor()

    cursor.execute("SELECT * FROM foo")

    for i in pg_wait(cursor.connection, env, 3):
        yield i

    print("query result available")

    for record in cursor:
        yield str(record)

    connection.close()



#!./uwsgi --http-socket :9090 --async 100 ...
# same chat example but using uwsgi async api
# for pypy + continulets just run:
# uwsgi --http-socket :9090 --pypy-home /opt/pypy --pypy-wsgi-file tests/websockets_chat_async.py --pypy-eval "uwsgi_pypy_setup_continulets()" --async 100
import uwsgi
import time
import redis
import sys


def application(env, sr):

    ws_scheme = 'ws'
    if 'HTTPS' in env or env['wsgi.url_scheme'] == 'https':
        ws_scheme = 'wss'

    if env['PATH_INFO'] == '/':
        sr('200 OK', [('Content-Type', 'text/html')])
        output = """
    <html>
      <head>
          <script language="Javascript">
            var s = new WebSocket("%s://%s/foobar/");
            s.onopen = function() {
              alert("connected !!!");
              s.send("ciao");
            };
            s.onmessage = function(e) {
                var bb = document.getElementById('blackboard')
                var html = bb.innerHTML;
                bb.innerHTML = html + '<br/>' + e.data;
            };

            s.onerror = function(e) {
                        alert(e);
                }

        s.onclose = function(e) {
                alert("connection closed");
        }

            function invia() {
              var value = document.getElementById('testo').value;
              s.send(value);
            }
          </script>
     </head>
    <body>
        <h1>WebSocket</h1>
        <input type="text" id="testo"/>
        <input type="button" value="invia" onClick="invia();"/>
        <div id="blackboard" style="width:640px;height:480px;background-color:black;color:white;border: solid 2px red;overflow:auto">
        </div>
    </body>
    </html>
        """ % (ws_scheme, env['HTTP_HOST'])
        if sys.version_info[0] > 2:
            return output.encode('latin1')
        return output
    elif env['PATH_INFO'] == '/favicon.ico':
        return ""
    elif env['PATH_INFO'] == '/foobar/':
        uwsgi.websocket_handshake(env['HTTP_SEC_WEBSOCKET_KEY'], env.get('HTTP_ORIGIN', ''))
        print("websockets...")
        r = redis.StrictRedis(host='localhost', port=6379, db=0)
        channel = r.pubsub()
        channel.subscribe('foobar')

        websocket_fd = uwsgi.connection_fd()
        redis_fd = channel.connection._sock.fileno()

        while True:
            uwsgi.wait_fd_read(websocket_fd, 3)
            uwsgi.wait_fd_read(redis_fd)
            uwsgi.suspend()
            fd = uwsgi.ready_fd()
            if fd > -1:
                if fd == websocket_fd:
                    msg = uwsgi.websocket_recv_nb()
                    if msg:
                        r.publish('foobar', msg)
                elif fd == redis_fd:
                    msg = channel.parse_response()
                    print(msg)
                    # only interested in user messages
                    t = 'message'
                    if sys.version_info[0] > 2:
                        t = b'message'
                    if msg[0] == t:
                        uwsgi.websocket_send("[%s] %s" % (time.time(), msg))
            else:
                # on timeout call websocket_recv_nb again to manage ping/pong
                msg = uwsgi.websocket_recv_nb()
                if msg:
                    r.publish('foobar', msg)



from uwsgidecorators import *
import gevent


@spool
def longtask(*args):
    print(args)
    return uwsgi.SPOOL_OK


def level2():
    longtask.spool(foo='bar', test1='test2')


def level1():
    gevent.spawn(level2)


def application(environ, start_response):
    start_response('200 OK', [('Content-Type', 'text/html')])

    gevent.spawn(level1)

    for i in range(100):
        yield "counter: %d<br/>" % i



# uwsgi --env foo=bar --env foo2=bar --http :8080 --wsgi-file t/python/testmultipleenv.py
import os

assert os.getenv('foo') == 'bar'
assert os.getenv('foo2') == 'bar'

print os.getenv('foo')
print os.getenv('foo2')

def application(e, sr):
    sr('200 OK', [('Content-Type', 'text/html')])
    return b'Hello world'



import uwsgi


def hello_signal(num, payload):
    print("i am the signal %d" % num)


def hello_signal2(num, payload):
    print("i am the signal %d with payload: %s" % (num, payload))


def hello_file(num, filename):
    print("file %s has been modified !!!" % filename)


def hello_timer(num, secs):
    print("%s seconds elapsed" % secs)

# uwsgi.register_signal(30, uwsgi.SIGNAL_KIND_WORKER, hello_signal)
uwsgi.register_signal(30, "workers", hello_signal)
uwsgi.register_signal(22, "worker", hello_signal2, "*** PAYLOAD FOO ***")

uwsgi.register_file_monitor(3, "/tmp", "workers", hello_file)
uwsgi.register_timer(26, 2, "worker", hello_timer)
uwsgi.register_timer(17, 4, "worker2", hello_timer)
uwsgi.register_timer(5, 8, "worker3", hello_timer)


def application(env, start_response):

    start_response('200 Ok', [('Content-Type', 'text/html')])

    # this will send a signal to the master that will report it to the first available worker
    uwsgi.signal(30)
    uwsgi.signal(22)

    return "signals sent to workers"



import unittest
import subprocess
import time

class SignalHandlingTest(unittest.TestCase):

    def test_int_mercyless(self):
        started_at = int(time.time())
        ret = subprocess.call(['./uwsgi', '--master', '--mule=t/mules/mulebrain.py', '--py-call-osafterfork', '--socket=:0', '--mule-reload-mercy=1'])
        self.assertEqual(ret, 0)
        self.assertLess(int(time.time()) - started_at, 6)

    def test_int(self):
        started_at = int(time.time())
        ret = subprocess.call(['./uwsgi', '--set=test_mule_timeout=3', '--master', '--mule=t/mules/mulebrain.py', '--py-call-osafterfork', '--socket=:0'])
        self.assertEqual(ret, 0)
        self.assertLess(int(time.time()) - started_at, 8)

    def test_hup(self):
        started_at = int(time.time())
        ret = subprocess.call(['./uwsgi', '--set=test_signal=1', '--set=test_mule_timeout=3', '--exit-on-reload', '--master', '--mule=t/mules/mulebrain.py', '--py-call-osafterfork', '--socket=:0'])
        self.assertEqual(ret, 0)
        self.assertLess(int(time.time()) - started_at, 8)

    def test_int_with_threads(self):
        started_at = int(time.time())
        ret = subprocess.call(['./uwsgi', '--enable-threads', '--set=test_mule_timeout=3', '--master', '--mule=t/mules/mulebrain.py', '--py-call-osafterfork', '--socket=:0'])
        self.assertEqual(ret, 0)
        self.assertLess(int(time.time()) - started_at, 8)
        

if __name__ == '__main__':
    unittest.main()



#! /usr/bin/env python
# coding = utf-8

import uwsgi
import unittest
import os
import fcntl
from shutil import rmtree
import time
from signal import signal, SIGINT
import spooler_priority_constants


def spoolersTaskList():
    # Get the list of tasks
    tasks = [
        os.path.join(s, fn)
        for s in uwsgi.spoolers
        for fn in os.listdir(s)
    ]

    for t in tasks[:]:
        if os.path.isdir(t):
            tasks += [os.path.join(t, fn) for fn in os.listdir(t)]
            tasks.remove(t)

    return tasks


def is_locked(filepath):
    # Check if file is locked
    with open(filepath, "a+") as f:
        try:
            fcntl.lockf(f.fileno(), fcntl.LOCK_EX | fcntl.LOCK_NB)
            is_locked = False
        except IOError:
            is_locked = True
    return is_locked


def lockedTasks(tasklist):
    # List of locked tasks
    return [fp for fp in spoolersTaskList() if is_locked(fp)]


def taskParameters(filepath):
    # Retrieve parameters
    return uwsgi.parsefile(filepath)


def cleanTasks():
    # Clean any remaining task
    for s in uwsgi.spoolers:
        for f in os.listdir(s):
            path = os.path.join(s, f)
            if os.path.isdir(path):
                rmtree(os.path.join(s, f))
            else:
                os.remove(path)


class BitmapTest(unittest.TestCase):

    def setUp(self):
        for priority, name in spooler_priority_constants.tasks:
            task = {'name': name, 'at': int(time.time() + 10)}
            if priority is not None:
                task['priority'] = str(priority)
            uwsgi.spool(task)

    def test_priority(self):
        uwsgi.signal_wait(17)
        print("Signal received.")

        with open(spooler_priority_constants.LOGFILE) as log:
            # Check logging ordering.
            loglines = [line.rstrip() for line in log]
            self.assertEqual(loglines, spooler_priority_constants.ordered_tasks)

signal(SIGINT, cleanTasks)
unittest.main()



import socket
import sys

from six.moves import input


s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

(addr, port) = sys.argv[1].split(':')
s.connect((addr, int(port)))

s.send("POST /send HTTP/1.1\r\n")
s.send("Transfer-Encoding: chunked\r\n\r\n")

while True:
    msg = input("msg >> ")
    s.send("%X\r\n%s\r\n" % (len(msg), msg))



#!/usr/bin/env python

import os
import sys
import shutil
from setuptools import setup, find_packages
from setuptools.command.install import install
from subprocess import check_call


data_dir = os.path.dirname(os.path.realpath(__file__))


def find_file(path):
    return os.path.join(data_dir, path)


class InstallCommand(install):
    """Custom install command that compiles uwsgi during installation."""

    description = "install envoxyd"

    def run(self):
        """Build uWSGI and install package."""
        # First run the normal install to set up paths
        install.run(self)
        
        # Now compile and install the binary
        setup_dir = os.path.dirname(os.path.abspath(__file__))
        uwsgi_src_dir = os.path.join(setup_dir, "uwsgi")
        
        if not os.path.exists(uwsgi_src_dir):
            raise RuntimeError(f"uwsgi source directory not found at {uwsgi_src_dir}")
        
        # Copy template files to uwsgi directory
        templates_dir = os.path.join(setup_dir, "envoxyd", "templates", "uwsgi")
        if os.path.exists(templates_dir):
            print(f"Copying uwsgi templates from {templates_dir}")
            for item in os.listdir(templates_dir):
                src = os.path.join(templates_dir, item)
                dst = os.path.join(uwsgi_src_dir, item)
                if os.path.isfile(src):
                    shutil.copy2(src, dst)
                elif os.path.isdir(src):
                    shutil.copytree(src, dst, dirs_exist_ok=True)
        
        # Also copy run.py to embed/
        run_py_src = os.path.join(setup_dir, "envoxyd", "templates", "run.py")
        if os.path.exists(run_py_src):
            embed_dir = os.path.join(uwsgi_src_dir, "embed")
            os.makedirs(embed_dir, exist_ok=True)
            shutil.copy2(run_py_src, os.path.join(embed_dir, "run.py"))
        
        # Compile uwsgi with flask profile
        print(f"Compiling uwsgi in {uwsgi_src_dir}...")
        python_executable = sys.executable
        check_call(
            [python_executable, "uwsgiconfig.py", "--build", "flask"],
            cwd=uwsgi_src_dir
        )
        
        # Find the built binary
        built_binary = None
        for binary_name in ["envoxyd", "uwsgi"]:
            src_binary = os.path.join(uwsgi_src_dir, binary_name)
            if os.path.exists(src_binary):
                built_binary = src_binary
                print(f"Found built binary: {built_binary}")
                break
        
        if not built_binary:
            raise RuntimeError("Built uwsgi binary not found")
        
        # Install to {prefix}/bin/ (works with venv, user install, system install)
        import sysconfig
        scripts_dir = sysconfig.get_path('scripts')
        dest_binary = os.path.join(scripts_dir, "envoxyd")
        
        print(f"Installing envoxyd to {dest_binary}")
        os.makedirs(scripts_dir, exist_ok=True)
        shutil.copy2(built_binary, dest_binary)
        os.chmod(dest_binary, 0o755)
        print("uwsgi compilation and installation complete")


packages = find_packages(include=["envoxyd", "envoxyd.*"], exclude=["tests", "uwsgi.build"])

"""
Resolve package metadata for envoxyd from local pyproject.toml.

Reads metadata from [project] section in vendors/pyproject.toml.
"""

# Read metadata from local pyproject.toml (in same directory as this setup.py)
_local_ppath = os.path.join(data_dir, 'pyproject.toml')

try:
    import tomllib as _toml  # Python 3.11+
except ImportError:
    import tomli as _toml  # type: ignore

if not os.path.exists(_local_ppath):
    raise ValueError(
        f"Missing pyproject.toml in {data_dir}. "
        "This file is required to build envoxyd."
    )

with open(_local_ppath, 'rb') as _f:
    _pdata = _toml.load(_f)
    _project_table = _pdata.get('project', {})

if not _project_table:
    raise ValueError(
        f"Missing [project] section in {_local_ppath}. "
        "This section is required to build envoxyd."
    )

# Extract metadata from project table
_name = _project_table["name"]
_version = _project_table["version"]
_description = _project_table["description"]
_requires_python = _project_table["requires-python"]
_dependencies = _project_table["dependencies"]

# Extract author info from authors array
_authors = _project_table.get("authors", [])
_author = _authors[0].get("name") if _authors else None
_author_email = _authors[0].get("email") if _authors else None

# Extract URL from project.urls
_urls = _project_table.get("urls", {})
_url = _urls.get("Homepage")

# License from classifiers
_project_license = "MIT"

# Prepare cmdclass with custom install command
cmdclass = {"install": InstallCommand}

setup(
    name=_name,
    version=_version,
    description=_description,
    author=_author,
    author_email=_author_email,
    url=_url,
    license=_project_license,
    packages=packages,
    install_requires=_dependencies,
    package_dir={
        "envoxyd": "envoxyd/",
    },
    package_data={
        "envoxyd": [
            "templates/run.py",
            "templates/view.py",
            "templates/confs/envoxy.json",
            "tools/*",
            "templates/__init__.py",
            "templates/confs/__init__.py",
        ]
    },
    scripts=["envoxyd/tools/envoxy-cli"],
    cmdclass=cmdclass,
    python_requires=_requires_python,
    include_package_data=True,
)




import sys

content_type = 'image/png'
filename = 'logo_uWSGI.png'

try:
    filename = sys.argv[1]
except IndexError:
    pass

try:
    content_type = sys.argv[2]
except IndexError:
    pass


def application(environ, start_response):
    start_response('200 OK', [('Content-Type', content_type)])
    fd = open(filename)
    yield environ['wsgi.file_wrapper'](fd, 32*1024)



from twisted.internet import defer, protocol, reactor
from twisted.protocols import basic
from twisted.web2 import http, resource, responsecode, stream

import struct


class uWSGIClientResource(resource.LeafResource):

    def __init__(self, app='', port=3030, host='localhost'):
        resource.LeafResource.__init__(self)
        self.host = host
        self.port = port
        self.app = app

    def renderHTTP(self, request):
        return uWSGI(request, self.app, self.host, self.port)


def uWSGI(request, app, host, port):
    if request.stream.length is None:
        return http.Response(responsecode.LENGTH_REQUIRED)

    request.uwsgi_app = app
    factory = uWSGIClientProtocolFactory(request)
    reactor.connectTCP(host, port, factory)
    return factory.deferred


class uWSGIClientProtocol(basic.LineReceiver):

    def __init__(self, request, deferred):
        self.request = request
        self.deferred = deferred
        self.stream = stream.ProducerStream()
        self.response = http.Response(stream=self.stream)
        self.status_parsed = None

    def build_uwsgi_var(self, key, value):
        return struct.pack('<H', len(key)) + key + struct.pack('<H', len(value)) + value

    def connectionMade(self):
        print(self.request.__dict__)
        # reset response parser
        self.status_parsed = None
        # build header and vars
        vars = ''

        if self.request.stream.length:
                vars += self.build_uwsgi_var('CONTENT_LENGTH', str(self.request.stream.length))

        for hkey, hval in self.request.headers.getAllRawHeaders():
                # use a list, probably it will be extended
                if hkey.lower() not in ('content-type'):
                        vars += self.build_uwsgi_var('HTTP_' + hkey.upper().replace('-', '_'), ','.join(hval))
                else:
                        vars += self.build_uwsgi_var(hkey.upper().replace('-', '_'), ','.join(hval))

        vars += self.build_uwsgi_var('REQUEST_METHOD', self.request.method)
        vars += self.build_uwsgi_var('SCRIPT_NAME', self.request.uwsgi_app)
        vars += self.build_uwsgi_var('PATH_INFO', self.request.path[len(self.request.uwsgi_app):])
        vars += self.build_uwsgi_var('QUERY_STRING', self.request.querystring)
        vars += self.build_uwsgi_var('SERVER_NAME', self.request.host)
        vars += self.build_uwsgi_var('SERVER_PORT', str(self.request.port))
        vars += self.build_uwsgi_var('SERVER_PROTOCOL', self.request.scheme.upper() + '/' + str(self.request.clientproto[0]) + '.' + str(self.request.clientproto[1]))

        vars += self.build_uwsgi_var('REQUEST_URI', self.request.uri)
        vars += self.build_uwsgi_var('REMOTE_ADDR', self.request.remoteAddr.host)

        self.transport.write(struct.pack('<BHB', 0, len(vars), 0))
        self.transport.write(vars)

        # send request data
        stream.StreamProducer(self.request.stream).beginProducing(self.transport)

    def lineReceived(self, line):
        if self.status_parsed is None:
                self.response.code = line.split(' ', 2)[1]
                self.status_parsed = True
                return

        # end of headers
        if line == '':
            self.setRawMode()
            self.deferred.callback(self.response)
            self.response = None
            return

        name, value = line.split(':', 1)
        value = value.strip()

        self.response.headers.addRawHeader(name, value)

    def rawDataReceived(self, data):
        self.stream.write(data)

    def connectionLost(self, reason):
        self.stream.finish()


class uWSGIClientProtocolFactory(protocol.ClientFactory):

    protocol = uWSGIClientProtocol
    noisy = False  # Make Factory shut up

    def __init__(self, request):
        self.request = request
        self.deferred = defer.Deferred()

    def buildProtocol(self, addr):
        return self.protocol(self.request, self.deferred)

    def clientConnectionFailed(self, connector, reason):
        self.sendFailureResponse(reason)

    def sendFailureResponse(self, reason):
        response = http.Response(code=responsecode.BAD_GATEWAY, stream=str(reason.value))
        self.deferred.callback(response)



#!./uwsgi --http-socket :9090  --http-raw-body --gevent 100 --module tests.websockets_chat_2
import uwsgi
import time

import gevent
from gevent.queue import Queue

class ClientManager(object):
    clients = set()

    @classmethod
    def add(cls, client):
        cls.clients.add(client)

    @classmethod
    def remove(cls, client):
        cls.clients.remove(client)

    @classmethod
    def count(cls):
        return len(cls.clients)

    @classmethod
    def broadcast(cls, data):
        data = "{0} {1}".format(time.time(), data)
        def do_broadcast():
            for c in cls.clients:
                c.send(data)

        gevent.spawn(do_broadcast)


class Client(object):
    def __init__(self):
        self.ctx = None
        self.send_queue = Queue()
        self.jobs = []


    def _recv_job(self):
        while True:
            data = uwsgi.websocket_recv(request_context=self.ctx)
            self.on_data(data)

    def _send_job(self):
        while True:
            data = self.send_queue.get()
            uwsgi.websocket_send(data, request_context=self.ctx)

    def _exit(self, *args):
        for j in self.jobs:
            j.unlink(self._exit)

        gevent.killall(self.jobs)
        ClientManager.remove(self)
        self.on_exit()


    def on_data(self, data):
        print "GOT: {0}".format(data)
        ClientManager.broadcast(data)


    def on_exit(self):
        print "bye bye..."


    def send(self, data):
        self.send_queue.put(data)


    def start(self):
        uwsgi.websocket_handshake()
        self.ctx = uwsgi.request_context()

        ClientManager.add(self)

        self.jobs.extend([
            gevent.spawn(self._recv_job),
            gevent.spawn(self._send_job),
        ])

        for j in self.jobs:
            j.link(self._exit)

        gevent.joinall(self.jobs)




def application(env, sr):

    ws_scheme = 'ws'
    if 'HTTPS' in env or env['wsgi.url_scheme'] == 'https':
        ws_scheme = 'wss'

    if env['PATH_INFO'] == '/':
        sr('200 OK', [('Content-Type', 'text/html')])
        return """
    <html>
      <head>
          <script language="Javascript">
            var s = new WebSocket("%s://%s/foobar/");
            s.onopen = function() {
              alert("connected !!!");
              s.send("ciao");
            };
            s.onmessage = function(e) {
                var bb = document.getElementById('blackboard')
                var html = bb.innerHTML;
                bb.innerHTML = html + '<br/>' + e.data;
            };

            s.onerror = function(e) {
                        alert(e);
                }

        s.onclose = function(e) {
                alert("connection closed");
        }

            function invia() {
              var value = document.getElementById('testo').value;
              s.send(value);
            }
          </script>
     </head>
    <body>
        <h1>WebSocket</h1>
        <input type="text" id="testo"/>
        <input type="button" value="invia" onClick="invia();"/>
        <div id="blackboard" style="width:640px;height:480px;background-color:black;color:white;border: solid 2px red;overflow:auto">
        </div>
    </body>
    </html>
        """ % (ws_scheme, env['HTTP_HOST'])
    elif env['PATH_INFO'] == '/favicon.ico':
        return ""
    elif env['PATH_INFO'] == '/foobar/':
        print "websockets..."
        client = Client()
        client.start()

        return ""




from uwsgidecorators import spool
from threading import Thread

from six.moves import queue

queues = {}


class queueconsumer(object):

    def __init__(self, name, num=1, **kwargs):
        self.name = name
        self.num = num
        self.queue = queue.Queue()
        self.threads = []
        self.func = None
        queues[self.name] = self

    @staticmethod
    def consumer(self):
        while True:
            req = self.queue.get()
            print(req)
            self.func(req)
            self.queue.task_done()

    def __call__(self, f):
        self.func = f
        for i in range(self.num):
            t = Thread(target=self.consumer, args=(self,))
            self.threads.append(t)
            t.daemon = True
            t.start()


@spool
def spooler_enqueuer(arguments):
    if 'queue' in arguments:
        queue = arguments['queue']
        queues[queue].queue.put(arguments)
    else:
        raise Exception("You have to specify a queue name")


def enqueue(*args, **kwargs):
    return spooler_enqueuer.spool(*args, **kwargs)
